define(["exports", "../utils.js"], function (_exports, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* istanbul ignore next: coverage is calculated in Chrome, this code is for IE */
  function getAnchorOrigin(anchor) {
    // IE11: on HTTP and HTTPS the default port is not included into
    // window.location.origin, so won't include it here either.
    var port = anchor.port;
    var protocol = anchor.protocol;
    var defaultHttp = protocol === 'http:' && port === '80';
    var defaultHttps = protocol === 'https:' && port === '443';
    var host = defaultHttp || defaultHttps ? anchor.hostname // does not include the port number (e.g. www.example.org)
    : anchor.host; // does include the port number (e.g. www.example.org:80)

    return "".concat(protocol, "//").concat(host);
  } // The list of checks is not complete:
  //  - SVG support is missing
  //  - the 'rel' attribute is not considered


  function vaadinRouterGlobalClickHandler(event) {
    // ignore the click if the default action is prevented
    if (event.defaultPrevented) {
      return;
    } // ignore the click if not with the primary mouse button


    if (event.button !== 0) {
      return;
    } // ignore the click if a modifier key is pressed


    if (event.shiftKey || event.ctrlKey || event.altKey || event.metaKey) {
      return;
    } // find the <a> element that the click is at (or within)


    var anchor = event.target;
    var path = event.composedPath ? event.composedPath() : event.path || []; // FIXME(web-padawan): `Symbol.iterator` used by webcomponentsjs is broken for arrays
    // example to check: `for...of` loop here throws the "Not yet implemented" error

    for (var i = 0; i < path.length; i++) {
      var target = path[i];

      if (target.nodeName && target.nodeName.toLowerCase() === 'a') {
        anchor = target;
        break;
      }
    }

    while (anchor && anchor.nodeName.toLowerCase() !== 'a') {
      anchor = anchor.parentNode;
    } // ignore the click if not at an <a> element


    if (!anchor || anchor.nodeName.toLowerCase() !== 'a') {
      return;
    } // ignore the click if the <a> element has a non-default target


    if (anchor.target && anchor.target.toLowerCase() !== '_self') {
      return;
    } // ignore the click if the <a> element has the 'download' attribute


    if (anchor.hasAttribute('download')) {
      return;
    } // ignore the click if the target URL is a fragment on the current page


    if (anchor.pathname === window.location.pathname && anchor.hash !== '') {
      return;
    } // ignore the click if the target is external to the app
    // In IE11 HTMLAnchorElement does not have the `origin` property


    var origin = anchor.origin || getAnchorOrigin(anchor);

    if (origin !== window.location.origin) {
      return;
    } // if none of the above, convert the click into a navigation event


    var _anchor = anchor,
        pathname = _anchor.pathname,
        search = _anchor.search,
        hash = _anchor.hash;

    if ((0, _utils.fireRouterEvent)('go', {
      pathname: pathname,
      search: search,
      hash: hash
    })) {
      event.preventDefault();
    }
  }
  /**
   * A navigation trigger for Vaadin Router that translated clicks on `<a>` links
   * into Vaadin Router navigation events.
   *
   * Only regular clicks on in-app links are translated (primary mouse button, no
   * modifier keys, the target href is within the app's URL space).
   *
   * @memberOf Router.Triggers
   * @type {NavigationTrigger}
   */


  var CLICK = {
    activate: function activate() {
      window.document.addEventListener('click', vaadinRouterGlobalClickHandler);
    },
    inactivate: function inactivate() {
      window.document.removeEventListener('click', vaadinRouterGlobalClickHandler);
    }
  };
  var _default = CLICK;
  _exports.default = _default;
});