define(["exports", "require", "../../../lit-element/lit-element.js"], function (_exports, _require, _litElement) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.LrndesignImagemapHotspot = void 0;
  _require = babelHelpers.interopRequireWildcard(_require);

  /**
   * `lrndesign-imagemap-hotspot`
   * creates an accessible image map
   * @demo demo/index.html
   * @customElement lrndesign-imagemap-hotspot
   */
  class LrndesignImagemapHotspot extends _litElement.LitElement {
    /**
     * LitElement constructable styles enhancement
     */
    static get styles() {
      return [_litElement.css`
        :host {
          display: none;
        }
        :host #desc {
          margin: 0 0 15px;
        }
        @media print {
          :host {
            display: block;
          }
        }
      `];
    }
    /**
     * LitElement render
     */


    render() {
      return _litElement.html`
      <relative-heading
        id="heading"
        ?hidden="${!this.label}"
        text="${this.label}"
      >
      </relative-heading>
      <div id="desc"><slot></slot></div>
    `;
    }
    /**
     * HTMLElement
     */


    constructor() {
      super();
      this.label = null;
      this.hotspotId = null;
      new Promise((res, rej) => _require.default(["../../relative-heading/relative-heading.js"], res, rej));
    }

    static get tag() {
      return "lrndesign-imagemap-hotspot";
    }

    static get properties() {
      return {
        /**
         * Label for the hotspot
         */
        label: {
          type: String
        },

        /**
         * Id of hotspot element inside the SVG
         */
        hotspotId: {
          type: String,
          attribute: "hotspot-id"
        }
      };
    }

    setParentHeading(parent) {
      this.shadowRoot.querySelector("#heading")._setParent(parent);
    }

  }

  _exports.LrndesignImagemapHotspot = LrndesignImagemapHotspot;
  window.customElements.define(LrndesignImagemapHotspot.tag, LrndesignImagemapHotspot);
});