/**
 * Copyright 2018 The Pennsylvania State University
 * @license Apache-2.0, see License.md for full text.
 */
import{html,PolymerElement}from"../../../@polymer/polymer/polymer-element.js";import{afterNextRender}from"../../../@polymer/polymer/lib/utils/render-status.js";import"../../../@polymer/polymer/lib/elements/dom-repeat.js";import"../../../@polymer/polymer/lib/elements/dom-if.js";import"../../../@polymer/iron-ajax/iron-ajax.js";import"../../simple-picker/simple-picker.js";import{ResponsiveUtilityBehaviors}from"../../responsive-utility/lib/responsive-utility-behaviors.js";import{displayBehaviors}from"./editable-table-behaviors.js";import"./editable-table-styles.js";class EditableTableDisplay extends(displayBehaviors(ResponsiveUtilityBehaviors(PolymerElement))){static get template(){return html`
      <style include="editable-table-styles">
        :host .th-or-td {
          padding: var(--editable-table-cell-padding);
        }
        :host([bordered]) .th {
          border: 1px solid var(--editable-table-border-color);
        }
        :host([striped]) .tbody-tr:nth-child(2n) .th,
        :host([striped]) .tbody-tr:nth-child(2n) .td {
          @apply --editable-table-style-stripe;
        }
        :host([sort]) thead th,
        :host([filter]) tbody td {
          padding: 0;
        }
        :host([column-header]) .thead-tr .th {
          @apply --editable-table-style-column-header;
        }
        :host([row-header]) .tbody-tr .th {
          @apply --editable-table-style-row-header;
        }
        :host([footer]) .tfoot-tr .th,
        :host([footer]) .tfoot-tr .td {
          @apply --editable-table-style-footer;
        }
        #column {
          width: calc(var(--simple-picker-option-size) + 6px);
          overflow: visible;
          display: none;
          margin-left: 10px;
          --simple-picker-border-width: 1px;
          --simple-picker-focus-border-width: 1px;
          --simple-picker-border-color: var(--editable-table-border-color);
        }
        @media screen {
          :host([responsive][responsive-size="xs"]) caption {
            padding: 0;
          }
          :host([responsive][responsive-size="xs"])
            caption
            > div
            > *:not(#column) {
            padding: 0 0 5px;
          }
          :host([responsive][responsive-size="xs"]) caption > div {
            display: flex;
            align-items: flex-end;
            justify-content: space-between;
          }
          :host([responsive][responsive-size="xs"]) #column {
            display: inline-flex;
          }
          :host([responsive][responsive-size="xs"]) .th[xs-hidden],
          :host([responsive][responsive-size="xs"]) .td[xs-hidden] {
            display: none;
          }
        }
      </style>
      <iron-ajax
        auto
        url="[[dataCsv]]"
        hidden$="[[!dataCsv]]"
        handle-as="text"
        debounce-duration="500"
        last-response="{{csvData}}"
        on-response="_loadExternalData"
      ></iron-ajax>
      <table id="table" class="table" hidden$="[[hidden]]">
        <caption>
          <div>
            [[caption]]
            <simple-picker
              id="column"
              align-right
              aria-labelledby$="[[tables.0.label]]"
              hide-sample
              value$="{{selected}}"
              on-change="_selectedChanged"
              options="[[options]]"
            >
            </simple-picker>
          </div>
        </caption>
        <thead hidden="[[!columnHeader]]">
          <tr class="tr thead-tr">
            <template
              is="dom-repeat"
              items="[[thead.0]]"
              as="th"
              index-as="index"
              mutable-data
              restamp
            >
              <th
                class="th th-or-td"
                cell-index$="[[index]]"
                numeric$="[[_isNumericColumn(index)]]"
                scope="col"
                xs-hidden$="[[_isColHidden(index,1)]]"
              >
                <template is="dom-if" if="[[sort]]" restamp>
                  <editable-table-sort
                    sort-column$="[[sortColumn]]"
                    column-index="[[index]]"
                    text$="[[_replaceBlankCell(th)]]"
                  ></editable-table-sort>
                </template>
                <template is="dom-if" if="[[!sort]]" restamp
                  >[[_replaceBlankCell(th)]]
                </template>
              </th>
            </template>
          </tr>
        </thead>
        <tbody id="tbody" class="tbody">
          <template
            is="dom-repeat"
            items="[[tbody]]"
            as="tr"
            filter="{{filterRows(filterColumn,filterText)}}"
            mutable-data
            restamp
          >
            <tr class="tr tbody-tr">
              <template
                is="dom-repeat"
                items="[[tr]]"
                as="cell"
                index-as="index"
                mutable-data
                restamp
              >
                <template
                  is="dom-if"
                  if="[[_isRowHeader(rowHeader,index)]]"
                  restamp
                >
                  <th
                    class="th th-or-td"
                    cell-index$="[[index]]"
                    numeric$="[[_isNumericColumn(index)]]"
                    xs-hidden$="[[_isColHidden(index,1)]]"
                    scope="row"
                  >
                    [[_replaceBlankCell(cell)]]
                  </th>
                </template>
                <template
                  is="dom-if"
                  if="[[!_isRowHeader(rowHeader,index)]]"
                  restamp
                >
                  <td
                    class="td cell th-or-td"
                    cell-index$="[[index]]"
                    numeric$="[[_isNumericColumn(index)]]"
                    negative$="[[_isNegative(cell)]]"
                    xs-hidden$="[[_isColHidden(index,1)]]"
                  >
                    <template is="dom-if" if="[[filter]]" restamp>
                      <editable-table-filter
                        column-index="[[index]]"
                        text$="[[_replaceBlankCell(cell)]]"
                        filtered$="[[_isFiltered(index,filterColumn,filtered)]]"
                      ></editable-table-filter>
                    </template>
                    <template is="dom-if" if="[[!filter]]" restamp>
                      <span class="cell">[[_replaceBlankCell(cell)]]</span>
                    </template>
                  </td>
                </template>
              </template>
            </tr>
          </template>
        </tbody>
        <template is="dom-if" if="[[footer]]">
          <tfoot class="tfoot">
            <tr class="tr tfoot-tr">
              <template
                is="dom-repeat"
                items="[[tfoot.0]]"
                as="cell"
                index-as="index"
                mutable-data
                restamp
              >
                <template is="dom-if" if="[[_isRowHeader(rowHeader,index)]]">
                  <th
                    class="th th-or-td"
                    cell-index$="[[index]]"
                    numeric$="[[_isNumericColumn(index)]]"
                    xs-hidden$="[[_isColHidden(index,1)]]"
                    scope="row"
                  >
                    [[_replaceBlankCell(cell)]]
                  </th>
                </template>
                <template is="dom-if" if="[[!_isRowHeader(rowHeader,index)]]">
                  <td
                    class="td cell th-or-td"
                    cell-index$="[[index]]"
                    numeric$="[[_isNumericColumn(index)]]"
                    negative$="[[_isNegative(cell)]]"
                    xs-hidden$="[[_isColHidden(index,1)]]"
                  >
                    [[_replaceBlankCell(cell)]]
                  </td>
                </template>
              </template>
            </tr>
          </tfoot>
        </template>
      </table>
      <div id="htmlImport" hidden><slot></slot></div>
    `}static get tag(){return"editable-table-display"}static get properties(){return{filterColumn:{type:Number,value:null},filtered:{type:Boolean,value:!1},filterText:{type:String,value:null},options:{type:Array,computed:"_getTheadOptions(thead)"},selected:{type:Number,value:1},sortMode:{type:String,value:"none"},sortColumn:{type:Number,value:-1},hidden:{type:Boolean,computed:"_hasNoData(data)"}}}_dataChanged(newValue,oldValue){if(!newValue||newValue.length<1||newValue[0].length<1){let table=this.children.item(0);null!=table&&"TABLE"===table.tagName&&this.importHTML(table)}this.dispatchEvent(new CustomEvent("change",{bubbles:!0,cancelable:!0,composed:!0,detail:newValue}))}_hasNoData(data){return!data||data.length<1||data[0].length<1}_getTheadOptions(thead){let temp=[];if(null!=thead&&thead.length>0)for(let i=1;i<thead[0].length;i++)temp.push([{alt:thead[0][i],value:i}]);return temp}_isColHidden(index,selected=1){return selected=selected||1,0!==parseInt(index)&&parseInt(index)!==parseInt(selected)}_isFiltered(column,filterColumn,filtered){return null!==filterColumn&&filterColumn===column&&filtered}_isNegative(cell){return this._isNumeric(cell)&&0===cell.trim().indexOf("-")}_isNumericColumn(index){let numeric=!0;for(let i=0;i<this.tbody.length;i++)this._isNumeric(this.tbody[i][index])||(numeric=!1);return numeric}_isRowHeader(rowHeader,index){return 0===index&&rowHeader}_tableChanged(){this._updateCols()}_selectedChanged(){this._updateCols()}_changeSortMode(e){this.sortColumn===e.detail.columnIndex&&"asc"===this.sortMode?this.sortMode="desc":this.sortColumn===e.detail.columnIndex&&"desc"===this.sortMode?this.sortMode="none":(this.sortMode="asc",this.sortColumn=e.detail.columnIndex),e.detail.setSortMode(this.sortMode),console.log("_changeSortMode",e.detail,this.sortMode),this.sortData(this.sortMode,e.detail.columnIndex)}_updateCols(){let selected=this.shadowRoot.querySelector("#column").value,cols=this.shadowRoot.querySelector("#table").querySelectorAll("th,td");if(cols.length>0)for(let i=0;i<cols.length;i++){let col=cols[i];this._isColHidden(col.cellIndex,selected)?col.setAttribute("xs-hidden",!0):col.removeAttribute("xs-hidden")}}filterRows(filterColumn,filterText){return null!=filterText?function(tr){return tr[filterColumn].toLowerCase().trim()===filterText.toLowerCase().trim()}:null}sortData(type,column){if("none"!==type&&!1!==type){let temp=this.tbody.slice();for(let i=0;i<temp.length;i++)temp[i].unshift(temp[i][column]);"asc"===type?temp.sort():temp.reverse();for(let i=0;i<temp.length;i++)this.set("tbody."+i,[]),this.set("tbody."+i,temp[i].slice(1));console.log("sortData",type,column,temp,this.data)}else console.log("sortData",type,column,this.data)}toggleFilter(e){void 0===e||this.filterColumn==e.detail.columnIndex&&this.filtered?(this.filtered=!1,this.filterText=null,this.filterColumn=null):(this.filterText=e.detail.text,this.filterColumn=e.detail.columnIndex,this.filtered=!0)}constructor(){super(),import("./editable-table-sort.js"),import("./editable-table-filter.js")}connectedCallback(){super.connectedCallback(),afterNextRender(this,(function(){this.addEventListener("change-sort-mode",this._changeSortMode.bind(this)),this.addEventListener("toggle-filter",this.toggleFilter.bind(this))}))}disconnectedCallback(){this.removeEventListener("change-sort-mode",this._changeSortMode.bind(this)),this.removeEventListener("toggle-filter",this.toggleFilter.bind(this)),super.disconnectedCallback()}}window.customElements.define(EditableTableDisplay.tag,EditableTableDisplay);export{EditableTableDisplay};